import {
  Component,
  OnInit,
  OnDestroy,
  inject,
  ComponentFactoryResolver,
  TemplateRef,
  ViewChild,
  ViewContainerRef,
  Renderer2,
  ElementRef,
} from '@angular/core';
import { Editor } from 'ngx-editor';
import { DateAdapter } from '@angular/material/core';
import { JournalEntry } from 'src/interfaces/interfaces';
import localeDe from '@angular/common/locales/de';
import { registerLocaleData } from '@angular/common';
import { DataService } from 'src/services/data/data.service';
import { TooltipService } from 'src/services/tooltip/tooltip.service';
import { HighlightComponent } from 'src/app/shared-components/highlight/highlight.component';
import { DomSanitizer } from '@angular/platform-browser';

@Component({
  selector: 'app-journal-notes',
  templateUrl: './journal-notes.component.html',
  styleUrl: './journal-notes.component.scss',
})
export class JournalNotesComponent implements OnInit, OnDestroy {
  @ViewChild('tooltip', { read: TemplateRef }) tooltip!: TemplateRef<any>;
  @ViewChild('test', { read: ViewContainerRef }) test!: ViewContainerRef;

  tooltipText = 'Initial';

  editor: Editor = new Editor();
  toolbar: any = [
    // default value
    ['bold', 'italic'],
    ['bullet_list'],
    [{ heading: ['h3', 'h4', 'h5', 'h6'] }],
  ];

  /** Used to show the interactale form or the display version of an entry. */
  public isInEditMode = false;

  /** The index of the currently active entry */
  public currentEntryIndex: number = 0;

  private backupIndex: number = -1;
  /** Indicates, if the currentEntry is a newly generated one that is still not saved. */
  public isNewEntry = false;

  /**The array of JournalEntries, synched to the   */
  public entries: JournalEntry[] = [];

  public tooltipifiedEntries: JournalEntry[] = [];

  tooltipifiedEntry: JournalEntry = {
    title: 'Title',
    content: 'Content',
    created: new Date(),
  };

  /** Holds the data for the current entry */
  public currentEntry: JournalEntry = {
    title: '',
    content: '',
    created: new Date(),
  };

  private _adapter: DateAdapter<any> = inject(DateAdapter);
  private dataService: DataService = inject(DataService);
  private tooltipService: TooltipService = inject(TooltipService);
  private sanitizer: DomSanitizer = inject(DomSanitizer);
  private renderer: Renderer2 = inject(Renderer2);
  private el: ElementRef = inject(ElementRef);

  ngOnInit(): void {
    registerLocaleData(localeDe);
    this._adapter.setLocale('de');
    this.entries = this.dataService.notesData;
    this.tooltipifiedEntries = this.entries;

    // if the list is empty, set the currentEntryIndex to -1 to hide the entry-container
    if (this.entries.length === 0) {
      this.currentEntryIndex = -1;
    } else {
      this.currentEntry = this.entries[0];
      this.tooltipifiedEntry = JSON.parse(JSON.stringify(this.currentEntry));
    }
    this.tooltipify();
  }

  /**
   * Sets the currentEntry variable when being clicked on in the entries list on the left.
   * @param index The index of the selected entry.
   */
  public selectEntry(index: number): void {
    this.currentEntryIndex = index;
    this.currentEntry = this.getEntryAt(index);
    this.isNewEntry = false;
    this.isInEditMode = false;
    // this.tooltipify();
    this.collectNamesFromEntry().forEach((name) => {
      this.addHighlightsToText(name);
    });
  }

  // DONE
  addEntry(): void {
    this.currentEntry = {
      title: '',
      content: '',
      created: new Date(),
    };
    this.isNewEntry = true;
    this.isInEditMode = true;
    this.backupIndex = this.currentEntryIndex;
    // Hightlight no entry because the placeholder is shown as active
    this.currentEntryIndex = -1;
  }

  public editEntry(): void {
    this.isInEditMode = true;
  }

  // DONE
  public saveEntry(): void {
    if (this.isNewEntry) {
      // Prepend the new JournalEntry
      this.entries.unshift(this.currentEntry);
      this.isNewEntry = false;
      this.currentEntryIndex = 0;
    }
    this.isInEditMode = false;
    this.entries[this.currentEntryIndex] = this.currentEntry;
    this.tooltipify();
    this.uploadNotes();
  }

  public discardEntry(): void {
    if (this.isNewEntry) {
      this.currentEntryIndex = this.backupIndex;
      this.isNewEntry = false;
    }
    if (this.entries.length > 0) {
      // Reset the currentEntry to the last saved version
      this.currentEntry = this.getEntryAt(this.currentEntryIndex);
    }
    this.isInEditMode = false;
  }

  deleteEntry(): void {
    this.entries.splice(this.currentEntryIndex, 1);
    if (this.entries.length === 0) {
      this.currentEntryIndex = -1;
    } else {
      this.currentEntryIndex = Math.max(this.currentEntryIndex - 1, 0);
      this.currentEntry = this.getEntryAt(this.currentEntryIndex);
    }
    this.uploadNotes();
  }

  private getEntryAt(index: number): JournalEntry {
    return JSON.parse(JSON.stringify(this.entries[index]));
  }

  private uploadNotes(): void {
    this.dataService.notesData = this.entries;
  }

  ////////////////////////////////////////////////////

  tooltipify() {
    let result: any = this.tooltipService.tooltipifyEntry(
      JSON.parse(JSON.stringify(this.currentEntry.content)),
    );
    // console.log(result);
    this.tooltipifiedEntry.content = result.content;

    // console.log(result.content);
    // console.log(this.tooltipifiedEntry);

    // TODO: AB hier funktioniert es noch nicht.
    setTimeout(() => {
      result.npcs.forEach((name: string) => {
        this.addHighlightsToText(name);
      });
    }, 1000);

    // result.forEach((entry: any) => {
    //   entry.content = this.sanitizer.bypassSecurityTrustHtml(entry.content);
    // });
    // this.tooltipifiedEntries = result;
  }

  /**
   * Searches the current entry for names that are mentioned in the text.
   * @returns An array of all the names that are mentioned in the current entry.
   */
  private collectNamesFromEntry(): string[] {
    const matches = this.currentEntry.content.match(/(?<=@)\w+/g);
    const uniqueMatches = [...new Set(matches)];
    return uniqueMatches;
  }

  /**
   * Highlights and adds tooltips to the names that are mentioned in the current entry.
   * @param name The name of the person which mentionings are to be highlighted.
   */
  addHighlightsToText(name: string) {
    console.log('Highlighting: ' + name);

    // Target the correct containers
    const parent = this.el.nativeElement.querySelectorAll('.' + name);
    console.log(parent);

    parent.forEach((element: any) => {
      const componentRef = this.test.createComponent(HighlightComponent);
      // The name
      componentRef.instance.text = name;
      // A reference to the tooltip template
      componentRef.instance.tooltip = this.tooltip;
      // Appends it at the right spot
      this.renderer.appendChild(element, componentRef.location.nativeElement);
    });
    // Modufies the text inside the tooltip template
    // Muss auf mouseover erfolgen
    this.tooltipText = 'Neuer Tooltip <b>text</b>';
  }

  ngOnDestroy(): void {
    this.editor.destroy();
  }
}
